import os
import re
from .syllables import count as count_syllables
from nltk.tokenize import sent_tokenize, TweetTokenizer
from nltk.stem.porter import PorterStemmer


class AnalyzerStatistics:
    def __init__(self, stats):
        self.stats = stats

    @property
    def num_poly_syllable_words(self):
        return self.stats['num_poly_syllable_words']

    @property
    def num_syllables(self):
        return self.stats['num_syllables']

    @property
    def num_letters(self):
        return self.stats['num_letters']

    @property
    def num_words(self):
        return self.stats['num_words']

    @property
    def num_sentences(self):
        return self.stats['num_sentences']

    @property
    def num_gunning_complex(self):
        return self.stats['num_gunning_complex']

    @property
    def num_dale_chall_complex(self):
        return self.stats['num_dale_chall_complex']

    @property
    def num_spache_complex(self):
        return self.stats['num_spache_complex']

    @property
    def avg_words_per_sentence(self):
        return self.num_words / self.num_sentences

    @property
    def avg_syllables_per_word(self):
        return self.num_syllables / self.num_words

    def __str__(self):
        return str(self.stats) + \
            ", avg_words_per_sentence " + str(self.avg_words_per_sentence) + \
            ", avg_syllables_per_word " + str(self.avg_syllables_per_word)


class Analyzer:
    def __init__(self):
        pass

    def analyze(self, text):
        self._dale_chall_set = self._load_dale_chall()
        self._spache_set = self._load_spache()
        stats = self._statistics(text)
        self.sentences = stats['sentences']  # hack for smog
        return AnalyzerStatistics(stats)

    def _statistics(self, text):
        tokens = self._tokenize(text)
        syllable_count = 0
        poly_syllable_count = 0
        word_count = 0
        letters_count = 0
        gunning_complex_count = 0
        dale_chall_complex_count = 0
        spache_complex_count = 0
        porter_stemmer = PorterStemmer()

        def is_gunning_complex(t, syllable_count):
            return syllable_count >= 3 and \
                not (self._is_proper_noun(t) or
                     self._is_compound_word(t))

        def is_dale_chall_complex(t):
            stem = porter_stemmer.stem(t.lower())
            return stem not in self._dale_chall_set

        def is_spache_complex(t):
            stem = porter_stemmer.stem(t.lower())
            return stem not in self._spache_set

        for t in tokens:

            if not self._is_punctuation(t):
                word_count += 1
                word_syllable_count = count_syllables(t)
                syllable_count += word_syllable_count
                letters_count += len(t)
                poly_syllable_count += 1 if word_syllable_count >= 3 else 0
                gunning_complex_count += \
                    1 if is_gunning_complex(t, word_syllable_count) \
                    else 0
                dale_chall_complex_count += \
                    1 if is_dale_chall_complex(t) else 0
                spache_complex_count += \
                    1 if is_spache_complex(t) else 0

        sentences = self._tokenize_sentences(text)
        sentence_count = len(sentences)

        return {
            'num_syllables': syllable_count,
            'num_poly_syllable_words': poly_syllable_count,
            'num_words': word_count,
            'num_sentences': sentence_count,
            'num_letters': letters_count,
            'num_gunning_complex': gunning_complex_count,
            'num_dale_chall_complex': dale_chall_complex_count,
            'num_spache_complex': spache_complex_count,
            'sentences': sentences,
        }

    def _tokenize_sentences(self, text):
        return sent_tokenize(text)

    def _tokenize(self, text):
        tokenizer = TweetTokenizer()
        return tokenizer.tokenize(text)

    def _is_punctuation(self, token):
        match = re.match('^[.,\/#!$%\'\^&\*;:{}=\-_`~()]$', token)
        return match is not None

    def _is_proper_noun(self, token):
        # pos = pos_tag(token)[0][1]
        # return pos == 'NNP'
        return token[0].isupper()

    def _is_compound_word(self, token):
        return re.match('.*[-].*', token) is not None

    def _load_dale_chall(self):
        file = 'dale_chall_porterstem.txt'
        cur_path = os.path.dirname(os.path.realpath(__file__))
        dale_chall_path = os.path.join(cur_path, '..', 'data', file)
        with open(dale_chall_path) as f:
            return set(line.strip() for line in f)

    def _load_spache(self):
        file = 'spache_easy_porterstem.txt'
        cur_path = os.path.dirname(os.path.realpath(__file__))
        spache_path = os.path.join(cur_path, '..', 'data', file)
        with open(spache_path) as f:
            return set(line.strip() for line in f)



# -*- coding: utf-8 -*-
"""
Scores the readability of a text and output a variety of metrics including: Flesch-Kincaid Grade Level, Gunning Fog, ARI, Dale Chall, SMOG, and more
"""
__title__ = 'py-readabilitiy-metrics'
__version__ = '1.4.5'
__author__ = 'Carmine DiMascio'
__license__ = 'MIT'
__copyright__ = 'Copyright 2019 Carmine DiMascio'

from .readability import Readability



from readability.exceptions import ReadabilityException


class Result:
    def __init__(self, score, grade_level):
        self.score = score
        self.grade_level = grade_level

    def __str__(self):
        return "score: {}, grade_level: '{}'". \
            format(self.score, self.grade_level)


class LinsearWrite:
    def __init__(self, stats):
        self._stats = stats
        if stats.num_words < 100:
            raise ReadabilityException('100 words required.')

    def score(self):
        score = self._score()
        return Result(
            score=score,
            grade_level=self._grade_level(score)
        )

    def _score(self):
        s = self._stats
        num_easy_words = s.num_words - s.num_poly_syllable_words
        num_hard_words = s.num_poly_syllable_words
        inter_score = (num_easy_words + (num_hard_words * 3)) / s.num_sentences
        if inter_score > 20:
            return inter_score / 2
        return (inter_score - 2) / 2

    def _grade_level(self, score):
        return str(round(score))



import unittest
from readability import Readability


class ReadabilityTest(unittest.TestCase):
    def setUp(self):
        text = """
        In linguistics, the Gunning fog index is a readability test for English writing. The index estimates the years of formal education a person needs to understand the text on the first reading. For instance, a fog index of 12 requires the reading level of a United States high school senior (around 18 years old). The test was developed in 1952 by Robert Gunning, an American businessman who had been involved in newspaper and textbook publishing.
        The fog index is commonly used to confirm that text can be read easily by the intended audience. Texts for a wide audience generally need a fog index less than 12. Texts requiring near-universal understanding generally need an index less than 8.
        """
        self.readability = Readability(text)

    def test_ari(self):
        r = self.readability.ari()
        print(r)
        self.assertEqual(9.551245421245422, r.score)
        self.assertEqual(['10'], r.grade_levels)
        self.assertEqual([15, 16], r.ages)

    def test_coleman_liau(self):
        r = self.readability.coleman_liau()
        print(r)
        self.assertEqual(10.673162393162393, r.score)
        self.assertEqual('11', r.grade_level)

    def test_dale_chall(self):
        r = self.readability.dale_chall()
        print(r)
        self.assertEqual(9.32399010989011, r.score)
        self.assertEqual(['college'], r.grade_levels)

    def test_flesch(self):
        r = self.readability.flesch()
        print(r)
        self.assertEqual(51.039230769230784, r.score)
        self.assertEqual(['10', '11', '12'], r.grade_levels)
        self.assertEqual('fairly_difficult', r.ease)

    def test_flesch_kincaid(self):
        r = self.readability.flesch_kincaid()
        print(r)
        self.assertEqual(10.125531135531137, r.score)
        self.assertEqual('10', r.grade_level)

    def test_gunning_fog(self):
        r = self.readability.gunning_fog()
        print(r)
        self.assertEqual(12.4976800976801, r.score)
        self.assertEqual('12', r.grade_level)

    def test_linsear_write(self):
        r = self.readability.linsear_write()
        print(r)
        self.assertEqual(11.214285714285714, r.score)
        self.assertEqual('11', r.grade_level)

    def test_smog(self):
        text = """
        In linguistics, the Gunning fog index is a readability test for English writing. The index estimates the years of formal education a person needs to understand the text on the first reading. For instance, a fog index of 12 requires the reading level of a United States high school senior (around 18 years old). The test was developed in 1952 by Robert Gunning, an American businessman who had been involved in newspaper and textbook publishing.
        The fog index is commonly used to confirm that text can be read easily by the intended audience. Texts for a wide audience generally need a fog index less than 12. Texts requiring near-universal understanding generally need an index less than 8.
        """
        text = ' '.join(text for i in range(0, 5))

        readability = Readability(text)

        #Test SMOG with 30 sentences
        r1 = readability.smog()
        
        #Test SMOG with all sentences
        r2 = readability.smog(all_sentences=True)


        print("all_sentences=False: %s ; all_sentences=True: %s" % (r1,r2))
        self.assertEqual(12.516099999999998, r1.score)
        self.assertEqual('13', r1.grade_level)

        self.assertEqual(12.785403640627713, r2.score)
        self.assertEqual('13', r2.grade_level)


    def test_spache(self):
        r = self.readability.spache()
        print(r)
        self.assertEqual(7.164945054945054, r.score)
        self.assertEqual('7', r.grade_level)

    def test_print_stats(self):
        stats = self.readability.statistics()
        self.assertEqual(562, stats['num_letters'])
        self.assertEqual(117, stats['num_words'])
        self.assertEqual(7, stats['num_sentences'])
        self.assertEqual(20, stats['num_polysyllabic_words'])




from .flesch import Flesch
from .flesch_kincaid import FleschKincaid
from .gunning_fog import GunningFog
from .coleman_liau import ColemanLiau
from .dale_chall import DaleChall
from .ari import ARI
from .linsear_write import LinsearWrite
from .smog import Smog
from .spache import Spache



from readability.exceptions import ReadabilityException


class Result:
    def __init__(self, score, grade_levels):
        self.score = score
        self.grade_levels = grade_levels

    def __str__(self):
        return "score: {}, grade_levels: {}". \
            format(self.score, self.grade_levels)


class DaleChall:
    def __init__(self, stats):
        self._stats = stats
        if stats.num_words < 100:
            raise ReadabilityException('100 words required.')

    def score(self):
        score = self._score()
        return Result(
            score=score,
            grade_levels=self._grade_levels(score))

    def _score(self):
        stats = self._stats
        words_per_sent = stats.num_words / stats.num_sentences
        percent_difficult_words = \
            stats.num_dale_chall_complex / stats.num_words * 100
        raw_score = 0.1579 * percent_difficult_words + 0.0496 * words_per_sent
        adjusted_score = raw_score + 3.6365 \
            if percent_difficult_words > 5 \
            else raw_score
        return adjusted_score

    def _grade_levels(self, score):
        if score <= 4.9:
            return ['1', '2', '3', '4']
        elif score >= 5 and score < 6:
            return ['5', '6']
        elif score >= 6 and score < 7:
            return ['7', '8']
        elif score >= 7 and score < 8:
            return ['9', '10']
        elif score >= 8 and score < 9:
            return ['11', '12']
        elif score >= 9 and score < 10:
            return ['college']
        else:
            return ['college_graduate']



from readability.exceptions import ReadabilityException


class Result:
    def __init__(self, score, grade_level):
        self.score = score
        self.grade_level = grade_level

    def __str__(self):
        return "score: {}, grade_level: '{}'". \
            format(self.score, self.grade_level)


class Spache:
    def __init__(self, stats):
        self._stats = stats
        if stats.num_words < 100:
            raise ReadabilityException('100 words required.')

    def score(self):
        score = self._score()
        return Result(
            score=score,
            grade_level=self._grade_level(score))

    def _score(self):
        stats = self._stats
        avg_sentence_len = stats.num_words / stats.num_sentences
        percent_difficult_words = \
            stats.num_spache_complex / stats.num_words * 100

        return (0.141 * avg_sentence_len) + (0.086 * percent_difficult_words) + 0.839

    def _grade_level(self, score):
        return str(round(score))
